use annotate_snippets::Renderer;
use annotate_snippets::{Level, Snippet, Message, Annotation};

/// An error formatter.
pub struct Error<'a> {
    snippet: Message<'a>,
}

impl<'a> Error<'a> {
    /// Create a new `Error` formatter.
    pub fn new(label: &'a str) -> Self {
        Self {
            snippet: Level::Error.title(label),
        }
    }

    /// Pass a new error to the formatter.
    pub fn error(
        mut self,
        line_start: usize,
        start: usize,
        end: usize,
        source: &'a str,
        label: &'a str,
    ) -> Self {
        self.snippet = self.snippet.snippet(Snippet
            ::source(source)
            .line_start(line_start)
            .fold(false)
            .annotations( vec![Level::Error
                .span(start.. end)
                .label(label)
            ])
        );
        self
    }

    /// Create a new footer.
    pub fn help(mut self, label: &'a str) -> Self {
        self.snippet = self.snippet.footer(
            Level::Help.title(label)
        );
        self
    }

    pub fn to_string(self) -> String {
        Renderer::styled().render(self.snippet).to_string()
    }
}
